#!/usr/bin/python
# -*- coding:UTF-8 -*-
#
#       File: gtkpimagizer.py
#       GUI of Pimagizer
#       
#       This file is part of Sherlock Downloader
#       Sherlock Downloader Copyright 2012-2013 Víctor Fernández Rico <vfrico@gmail.com>
#
#       Sherlock Downloader is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Sherlock Downloader is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from gi.repository import Gtk, Gdk, GdkPixbuf
from pimagizer import info
import math, os, glob
from PIL import Image
import gettext
import config

# For translations:
APP="pimagizer"
DIR="/usr/share/pimagizer/i18n"
# Esto permite traducir los textos escritos en el .py (no en glade)
gettext.textdomain(APP)
gettext.bindtextdomain(APP, DIR)
# Y con esto podemos marcar las cadenas a traducir de la forma _("cadena")
_ = gettext.gettext
# End translations 

class Pimagizer:
    def __init__(self,imputfile):
        # Crea la ventana de trabajo Principal y obtiene los objetos en Glade
        self.builder = Gtk.Builder()
        self.builder.add_from_file(
            "/usr/share/pimagizer/pimagizer.glade")
        # Y las siguientes 2 lineas permiten traducir los textos del Glade
        self.builder.set_translation_domain(APP)
        print "translation domain:" , self.builder.get_translation_domain()

        # Get main window and show
        self.window1 = self.builder.get_object("window1")
        self.window1.show()
        
        #~ # Puts one of them active
        #~ self.imgbox = self.builder.get_object("imgbox")
        #~ self.filebox = self.builder.get_object("filebox")
        #~ self.imgbox.hide()
        
        # Get Image
        #~ self.imgfile = self.builder.get_object("imagefile")
        self.wdgtimg = self.builder.get_object("mainimage")
        self.filename = "/usr/share/pimagizer/pimagizer-main.png"
        if imputfile == []:
            self.wdgtimg.set_from_pixbuf(self.resizing(self.filename))
            self.hidelabel = False
        else:
            self.filename = imputfile[0]
            self.nombre = str(imputfile[0].split("/")[-1:][0]) #Coge el nombre del archivo
            self.wdgtimg.set_from_pixbuf(self.resizing(imputfile[0]))
            self.hidelabel = True
        
        imagen = Image.open(self.filename)
        (width, height) = imagen.size
        
        # Image widget
        
        #~ self.wdgtimg.hide()
        #~ self.isimgbox = False
        
        #File chooser
        self.filech = self.builder.get_object("filechooserdialog1")
        
        # Get width y height
        self.inputwidth = self.builder.get_object("width")
        #~ self.adjw = Gtk.Adjustment(value=0, lower=1 , upper=9999)
        #~ self.inputwidth.set_adjustment(self.adjw)
        self.inputwidth.set_value(width)
        self.txtwidth = ""
        self.inputheight = self.builder.get_object("height")
        self.inputheight.set_value(height)
        #~ self.adjh = Gtk.Adjustment(value=0, lower=1, upper=9999)
        #~ self.inputheight.set_adjustment(self.adjh)
        self.txtheight = ""
        self.checkprop = self.builder.get_object("checkprop")
        self.checkprop.set_label(_(self.checkprop.get_label()))
        self.lblresol = self.builder.get_object("lblresol")
        self.lblresol.set_text(str(width)+"x"+str(height))
        self.lblst = self.builder.get_object("labelst")
        
        # Save folder
        self.btflsave = self.builder.get_object("filechooserbutton1")
        self.labelsave = self.builder.get_object("savefile")
        placeholder_text = _("file.png")
        self.labelsave.set_placeholder_text(placeholder_text)
        if self.hidelabel:
            self.labelsave.set_text(self.nombre)
            self.btflsave.set_current_folder(os.path.dirname(self.filename)) #Sets folder on Filechooserbutton
        self.selected = False
        
        # Preferencias
        self.wpref = self.builder.get_object("preferences")
        self.ntbkpref = self.builder.get_object("notebook1")
        
        # Formats box
        self.boxformats = self.builder.get_object("formatbox")

        # Translates
        self.label6 = self.builder.get_object("label6") # "Click on image to change"
        self.label6.set_text(_(self.label6.get_text()))
        if self.hidelabel:
            self.label6.hide() #Hides the label "Click on image to change"
        self.label3 = self.builder.get_object("label3") # "Save:"
        self.label3.set_text(_(self.label3.get_text()))
        self.label1 = self.builder.get_object("label1") # "Size:"
        self.label1.set_text(_(self.label1.get_text()))
        self.labelnwfl = self.builder.get_object("labelnwfl") # Is actived nw file or not?
        self.f_labelnwfl()
        #~ self.inputwidth.set_placeholder_text(_("Width"))
        #~ self.inputheight.set_placeholder_text(_("Height"))
        
        # On Preferences window
        self.label10 = self.builder.get_object("label10") # Right now, Pimagizer suports JPG and PNG formats
        self.label10.set_text(_(self.label10.get_text()))
        self.label8 = self.builder.get_object("label8") # Works with
        self.label8.set_text(_(self.label8.get_text()))
        self.label7 = self.builder.get_object("label7") # Launchpad
        self.label7.set_text(_(self.label7.get_text()))
        self.linkbutton1 = self.builder.get_object("linkbutton1") # Translate
        self.linkbutton1.set_label(_(self.linkbutton1.get_label()))
        self.linkbutton2 = self.builder.get_object("linkbutton2") # Bugs
        self.linkbutton2.set_label(_(self.linkbutton2.get_label()))
        self.linkbutton3 = self.builder.get_object("linkbutton3") # Questions
        self.linkbutton3.set_label(_(self.linkbutton3.get_label()))
        self.label9 = self.builder.get_object("label9") # Size of preview
        self.label9.set_text(_(self.label9.get_text()))
        self.label13 = self.builder.get_object("label13") # Here you can set the size of the image preview on main window
        self.label13.set_text(_(self.label13.get_text()))
        self.label12 = self.builder.get_object("label12") # Is recommended to set a value near to 300px
        self.label12.set_text(_(self.label12.get_text()))
        self.label11 = self.builder.get_object("label11") # Height
        self.label11.set_text(_(self.label11.get_text()))
        self.label5 = self.builder.get_object("label5") # Interface
        self.label5.set_text(_(self.label5.get_text()))
        #Options:saving
        self.switch_newfilename = self.builder.get_object("switch1") #Switch for new file name
        self.label15 = self.builder.get_object("label15") #label on tab
        self.label15.set_text(_(self.label15.get_text()))
        self.label16 = self.builder.get_object("label16") #title frame: <b>Saving options</b>
        self.label16.set_text(_(self.label16.get_text()))
        self.label16.set_use_markup(True)
        self.label17 = self.builder.get_object("label17") # Do you want to save the new file with a 
        self.label17.set_text(_(self.label17.get_text())) # new name as image(widthxheight).jpg?
        self.label18 = self.builder.get_object("label18") # save images with new name
        self.label18.set_text(_(self.label18.get_text()))
        self.label19 = self.builder.get_object("label19") # If you <b>turn off</b> this option, you will  
        print self.label19.get_text()
        self.label19.set_text(_(self.label19.get_text())) # <b>overwrite</b> all you save with Pimagizer
        self.label19.set_use_markup(True)

        self.label20 = self.builder.get_object("label20")
        self.label20.set_text(_(self.label20.get_text()))
        self.label21 = self.builder.get_object("label21")
        self.label21.set_text(_(self.label21.get_text()))
        
        # Preview size
        self.spinprew = self.builder.get_object("spinbutton1")
        self.spinprew.set_value(config.get_value("height"))
        signals = {
                "about_activate" : self.showabout,
                "main_close" : self.cerrarapp,
                "resize-accept" : self.resizeimg,
                "File-update" : self.updateimg,
                "main-update" : self.updateimg,
                "img-click" : self.updateimg,
                "height-cursor" : self.heightcursor,
                "width-cursor" : self.widthcursor,
                "filech-accept" : self.setfilename,
                "filech-cancel" : self.closefilename,
                "chfile" : self.changefile,
                "activeprop" : self.proporcionar,
                "show-preferences" : self.showpreferences,
                "prefer-cancel": self.cancelpref,
                "prefer-acept": self.aceptpref,
                "nwfile-lbl" : self.pref_saving,
                #"newnameact": self.newname
                }

        self.builder.connect_signals(signals)
        if(self.window1):
            self.window1.connect("destroy", self.cerrarapp)
    def cerrarapp(self,widget):
        print _("Closing program")
        Gtk.main_quit()
    
    def showabout(self,widget):
        print _("About")
        self.aboutwindow = self.builder.get_object("aboutdialog")
        self.aboutwindow.set_version(info.version)
        self.aboutwindow.run()
        self.aboutwindow.hide()
        
    def closeabout(self,widget):
        print _("Close about dialog")
        self.aboutwindow.hide()
        
    def resizeimg(self,widget):
        "Saves the image"
        imgfile, ext = os.path.splitext(self.filename)

        image = Image.open(self.filename)
        width = int(float(self.inputwidth.get_value_as_int()))
        height = int(float(self.inputheight.get_value_as_int()))
        image = image.resize((width,height), Image.ANTIALIAS)
        if(ext == ".jpg" or ext == ".JPG" or ext == ".jpeg" or ext == ".JPEG"):
            ext2 = "JPEG"
        elif(ext == ".png" or ext == ".PNG"):
            ext2 = "PNG"
        else:
            print "No format supported"
        imgname = self.labelsave.get_text() #Obtener el nombre deseado
        if self.folderset()[0]: #choosed carpeta
            final = self.folderset()[1]+"/"+imgname
            print final
        else:
            final = os.path.dirname(os.path.abspath(self.filename))+"/"+imgname #Nombre final de la dirección de guardado
        print "Saving",final
        self.lblst.set_text(_("Saving image as")+' <i>'+imgname+'</i>')
        self.lblst.set_use_markup(True)
        image.save(final)
        # if (self.nwnamefile() == False):
        #     print "Overwrite"
        #     print "Saving as",self.filename
        #     image.save(self.filename, ext2)
        # else:
        #     print "Saving as",imgfile+"("+self.lblresol.get_text()+")"+ext
        #     image.save(imgfile+"("+self.lblresol.get_text()+")"+ext, ext2)
        
    def resizing (self,filename):
        pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
        pbuf_width = pixbuf.get_width()
        pbuf_height = pixbuf.get_height()
        ratio = int(pbuf_width) * math.pow(int(pbuf_height),-1) # Division
        self.ratio = ratio
        height = config.get_value('height')
        width = height*ratio
        scaled_buf = pixbuf.scale_simple(width,height,GdkPixbuf.InterpType.BILINEAR)
        return scaled_buf
        
    def updateimg (self,widget):
        """Update image on window"""
        filename = self.imgfile.get_filename() # Variable filename contains path
        self.filename = filename
        self.wdgtimg.set_from_pixbuf(self.resizing(filename))
        if (self.isimgbox):
            self.wdgtimg.hide()
            self.isimgbox = False
            self.imgfile.show()
        else:
            self.imgfile.hide()
            self.wdgtimg.show()
            self.isimgbox = True
    
    def changefile (self,widget):
        self.filech.show()
        self.filech.run()
        
    def setfilename(self,widget):
        "After selecting a file on Filechooserbutton"
        self.filech.hide()
        self.filename = self.filech.get_filename()
        print "new image selected",self.filename
        self.wdgtimg.set_from_pixbuf(self.resizing(self.filename)) #Changes image
        self.btflsave.set_current_folder(os.path.dirname(self.filename)) #Sets folder on Filechooserbutton
        self.label6.hide() #Hides the label "Click on image to change"
        imagen = Image.open(self.filename)
        (width, height) = imagen.size #Gets width and height
        self.lblresol.set_text(str(width)+"x"+str(height)) #Sets text on top label
        self.inputheight.set_value(height) #set height on imput
        self.inputwidth.set_value(width) #set width on input
        self.setlabelsave() #Changes the imput text for file to save
        
    def setlabelsave (self):
        "Changes imput text (on the bottom) for change file"
        name , ext = os.path.splitext(os.path.basename(self.filename))
        if (self.nwnamefile() == False):
            self.labelsave.set_text(name+''+ext)
        else:
            self.labelsave.set_text(name+"("+self.lblresol.get_text()+")"+ext)
    
    def closefilename(self,widget):
        self.filech.hide()
    
    def calcheight(self):
        width = self.inputwidth.get_value_as_int()
        if width != 0:
            height = width/self.ratio
            #print "nuevo height", height
        else:
            #~ print "Valor 0"
            height = 1
        return int(height)
        
    def calcwidth(self):
        height = self.inputheight.get_value_as_int()
        if height != 0:
            width = height*self.ratio
            #print "nuevo width",width
        else:
            #~ print "Valor 0"
            width =1
        return int(width)
        
    def widthcursor(self,widget, *args):
        if(self.checkprop.get_active()):
            #print "New width"
            self.inputheight.set_value(self.calcheight())
        nwdth = float(self.inputwidth.get_value_as_int())
        nhght = float(self.inputheight.get_value_as_int())
        self.lblresol.set_text(str(int(nwdth))+"x"+str(int(nhght)))
        self.setlabelsave()
        self.ad = False

        #~ None
    def heightcursor(self,widget, *args):
        if(self.checkprop.get_active()):
            #print "New height"
            self.inputwidth.set_value(self.calcwidth())
        nwdth = float(self.inputwidth.get_value_as_int())
        nhght = float(self.inputheight.get_value_as_int())
        self.lblresol.set_text(str(int(nwdth))+"x"+str(int(nhght)))
        self.setlabelsave()
        self.da = False
        #~ None
    def proporcionar(self,widget):
        None
        
    def showpreferences (self,widget):
        "Shows preferences window"
        value = self.nwnamefile()
        print value
        self.switch_newfilename.set_active(value)
        self.wpref.show()
        #~ config.get_value("height")
    
    def cancelpref (self,widget):
        self.wpref.hide()
    
    def aceptpref (self,widget):
        "After clicking button accept on preferences window"
        self.wpref.hide()
        #Configure height
        config.set_value("height",self.spinprew.get_value_as_int())
        #Configure saving options
        if self.switch_newfilename.get_active():
            valor = 1 #New file name
        else:
            valor = 0 #overwrite
        print "Setting up value:",valor
        config.set_value("newname",valor) 
        self.f_labelnwfl()

    def nwnamefile(self): #returns true if file name is image(WxH).png
        if config.get_value("newname") == 0:
            #print "sobrescribir"
            return False #overwrite
        else:
            #print "No sobreescribe"
            return True #New file name
    def f_labelnwfl(self):
        print "New file"
        if self.nwnamefile():
            texto = _("Saving under new name")
        else:
            texto = _("You are <b>overwritting</b> the image file")
        self.labelnwfl.set_text(texto)
        self.labelnwfl.set_use_markup(True)
    def pref_saving(self,widget,*args):
        # notebook.set_tab_pos
        "Change position of tabs in notebook (set 1 as default)"
        posini = self.ntbkpref.get_current_page()
        movim = 1-posini
        if movim > 0: # + (plus)
            for i in range(0,movim):
                self.ntbkpref.next_page()
        elif movim < 0: # - (neg)
            movim = abs(movim)
            for i in range(0,movim):
                self.ntbkpref.prev_page()
        self.showpreferences(self.wpref)
    def folderset(self):
        """Returns value of filechooser if selected folder"""
        #if True:#self.selected:
        print "existe"
        return [True,self.btflsave.get_filename()]
        # else:
        #     print "no"
        #     return [False,""]


#~ #Ejecucion del programa
if __name__ == '__main__':
    pimagizer = Pimagizer([])
    Gtk.main()  
